@@ -1,25 +1,23 @@
 /*
   Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
 
-  The MySQL Connector/J is licensed under the terms of the GPL,
-  like most MySQL Connectors. There are special exceptions to the
-  terms and conditions of the GPL as it is applied to this software,
-  see the FLOSS License Exception available on mysql.com.
+  The MySQL Connector/J is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
+  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
+  this software, see the FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License as
-  published by the Free Software Foundation; version 2 of the
-  License.
+  This program is free software; you can redistribute it and/or modify it under the terms
+  of the GNU General Public License as published by the Free Software Foundation; version 2
+  of the License.
 
-  This program is distributed in the hope that it will be useful,  
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02110-1301 USA
+  You should have received a copy of the GNU General Public License along with this
+  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
+  Floor, Boston, MA 02110-1301  USA
  
  */
 package com.mysql.jdbc;
@@ -444,6 +442,7 @@ public class CallableStatement extends PreparedStatement implements
 	private final static String PARAMETER_NAMESPACE_PREFIX = "@com_mysql_jdbc_outparam_"; //$NON-NLS-1$
 
 	private static String mangleParameterName(String origParameterName) {
+		//Fixed for 5.5+ in callers
 		if (origParameterName == null) {
 			return null;
 		}
@@ -481,7 +480,7 @@ public class CallableStatement extends PreparedStatement implements
 	protected CallableStatementParamInfo paramInfo;
 
 	private CallableStatementParam returnValueParam;
-
+	
 	/**
 	 * Creates a new CallableStatement
 	 * 
@@ -599,7 +598,7 @@ public class CallableStatement extends PreparedStatement implements
 			}
 		}
 	}
-
+	
 	/**
 	 * Creates a new CallableStatement
 	 * 
@@ -807,31 +806,61 @@ public class CallableStatement extends PreparedStatement implements
 	}
 	
 	private void determineParameterTypes() throws SQLException {
-		if (this.connection.getNoAccessToProcedureBodies()) {
-			fakeParameterTypes(true);
-			
-			return;
-		}
 		
 		java.sql.ResultSet paramTypesRs = null;
 
 		try {
+			//Bug#57022, we need to check for db.SPname notation first
+			//  and pass on only SPname
 			String procName = extractProcedureName();
-
+			String quotedId = "";
+			try {
+				quotedId = this.connection.supportsQuotedIdentifiers() ? 
+						this.connection.getMetaData().getIdentifierQuoteString()	: "";
+			} catch (SQLException sqlEx) {
+				// Forced by API, never thrown from getIdentifierQuoteString() in
+				// this implementation.
+				AssertionFailedException.shouldNotHappen(sqlEx);
+			}
+			
+			List parseList = StringUtils.splitDBdotName(procName, "", 
+					quotedId , this.connection.isNoBackslashEscapesSet());
+			String tmpCatalog = "";
+			//There *should* be 2 rows, if any.
+			if (parseList.size() == 2) {
+				tmpCatalog = (String) parseList.get(0);
+				procName = (String) parseList.get(1);			
+			} else {
+				//keep values as they are
+			}
+			
 			java.sql.DatabaseMetaData dbmd = this.connection.getMetaData();
 
 			boolean useCatalog = false;
 
-			if (procName.indexOf(".") == -1) {
+			if (tmpCatalog.length() <= 0) {
 				useCatalog = true;
 			}
-
+			
 			paramTypesRs = dbmd.getProcedureColumns(this.connection
 					.versionMeetsMinimum(5, 0, 2)
-					&& useCatalog ? this.currentCatalog : null, null, procName,
+					&& useCatalog ? this.currentCatalog : tmpCatalog/*null*/, null, procName,
 					"%"); //$NON-NLS-1$
-
-			convertGetProcedureColumnsToInternalDescriptors(paramTypesRs);
+			
+			boolean hasResults = false;
+			try {
+				if (paramTypesRs.next()) {
+					paramTypesRs.previous();
+					hasResults = true;
+				}
+			} catch (Exception e) {
+				// paramTypesRs is empty, proceed with fake params. swallow, was expected 
+			}
+			if (hasResults){
+				convertGetProcedureColumnsToInternalDescriptors(paramTypesRs);
+			} else {
+				fakeParameterTypes(true);
+			}
 		} finally {
 			SQLException sqlExRethrow = null;
 
@@ -1001,13 +1030,18 @@ public class CallableStatement extends PreparedStatement implements
 	 *             if the parameter name is null or empty.
 	 */
 	protected String fixParameterName(String paramNameIn) throws SQLException {
-		if ((paramNameIn == null) || (paramNameIn.length() == 0)) {
+		//Fixed for 5.5+
+		if (((paramNameIn == null) || (paramNameIn.length() == 0)) && (!hasParametersView())) {
 			throw SQLError.createSQLException(
 					((Messages.getString("CallableStatement.0") + paramNameIn) == null) //$NON-NLS-1$
 							? Messages.getString("CallableStatement.15") : Messages.getString("CallableStatement.16"), SQLError.SQL_STATE_ILLEGAL_ARGUMENT, 
 									getExceptionInterceptor()); //$NON-NLS-1$ //$NON-NLS-2$
 		}
 
+		if ((paramNameIn == null) && (hasParametersView())) {
+			paramNameIn = "nullpn";
+		};
+
 		if (this.connection.getNoAccessToProcedureBodies()) {
 			throw SQLError.createSQLException("No access to parameters by name when connection has been configured not to access procedure bodies",
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, getExceptionInterceptor());
@@ -1434,6 +1468,7 @@ public class CallableStatement extends PreparedStatement implements
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, getExceptionInterceptor());
 		}
 		
+		//Fixed for 5.5+ in callers
 		if ((paramName == null) || (paramName.length() == 0)) {
 			throw SQLError.createSQLException(Messages.getString("CallableStatement.2"), //$NON-NLS-1$
 					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, getExceptionInterceptor());
@@ -1925,6 +1960,10 @@ public class CallableStatement extends PreparedStatement implements
 
 					this.parameterIndexToRsIndex[retrParamInfo.index] = localParamIndex++;
 
+					if ((retrParamInfo.paramName == null) && (hasParametersView())) {
+						retrParamInfo.paramName = "nullnp" + retrParamInfo.index;
+					}
+					
 					String outParameterName = mangleParameterName(retrParamInfo.paramName);
 
 					if (!firstParam) {
@@ -2072,7 +2111,12 @@ public class CallableStatement extends PreparedStatement implements
 				CallableStatementParam inParamInfo = (CallableStatementParam) paramIter
 						.next();
 
+				//Fix for 5.5+
 				if (inParamInfo.isOut && inParamInfo.isIn) {
+					if ((inParamInfo.paramName == null) && (hasParametersView())) {
+						inParamInfo.paramName = "nullnp" + inParamInfo.index;
+					};
+					
 					String inOutParameterName = mangleParameterName(inParamInfo.paramName);
 					StringBuffer queryBuf = new StringBuffer(
 							4 + inOutParameterName.length() + 1 + 1);
@@ -2199,6 +2243,11 @@ public class CallableStatement extends PreparedStatement implements
 						.next();
 
 				if (!this.callingStoredFunction && outParamInfo.isOut) {
+
+					if ((outParamInfo.paramName == null) && (hasParametersView())) {
+						outParamInfo.paramName = "nullnp" + outParamInfo.index;
+					};
+
 					String outParameterName = mangleParameterName(outParamInfo.paramName);
 
 					int outParamIndex = 0;
@@ -2461,6 +2510,20 @@ public class CallableStatement extends PreparedStatement implements
 	protected boolean checkReadOnlySafeStatement() throws SQLException {
 		return (super.checkReadOnlySafeStatement() || this.checkReadOnlyProcedure());
 	}
+	
+	private boolean hasParametersView() throws SQLException {
+		try {
+			if (this.connection.versionMeetsMinimum(5, 5, 0)) {
+				java.sql.DatabaseMetaData dbmd1 = new DatabaseMetaDataUsingInfoSchema(this.connection, this.connection.getCatalog());
+				return ((DatabaseMetaDataUsingInfoSchema)dbmd1).gethasParametersView();
+			} else {
+				return false;
+			}
+		} catch (SQLException e) {
+			return false;
+		}
+	}
 
 
+   
 }
