@@ -1,25 +1,23 @@
 /*
   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
 
-  The MySQL Connector/J is licensed under the terms of the GPL,
-  like most MySQL Connectors. There are special exceptions to the
-  terms and conditions of the GPL as it is applied to this software,
-  see the FLOSS License Exception available on mysql.com.
+  The MySQL Connector/J is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
+  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
+  this software, see the FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License as
-  published by the Free Software Foundation; version 2 of the
-  License.
+  This program is free software; you can redistribute it and/or modify it under the terms
+  of the GNU General Public License as published by the Free Software Foundation; version 2
+  of the License.
 
-  This program is distributed in the hope that it will be useful,  
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02110-1301 USA
+  You should have received a copy of the GNU General Public License along with this
+  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
+  Floor, Boston, MA 02110-1301  USA
 
  */
 package com.mysql.jdbc;
@@ -27,6 +25,7 @@ package com.mysql.jdbc;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Types;
+import java.util.List;
 
 /**
  * DatabaseMetaData implementation that uses INFORMATION_SCHEMA available in
@@ -38,7 +37,7 @@ import java.sql.Types;
 public class DatabaseMetaDataUsingInfoSchema extends DatabaseMetaData {
 
 	private boolean hasReferentialConstraintsView;
-	private boolean hasParametersView;
+	private final boolean hasParametersView;
 	
    protected DatabaseMetaDataUsingInfoSchema(MySQLConnection connToSet,
 			String databaseToSet) throws SQLException {
@@ -1467,6 +1466,16 @@ public class DatabaseMetaDataUsingInfoSchema extends DatabaseMetaData {
 			}
 		}
 
+		final String tableNamePat;
+		List parseList = StringUtils.splitDBdotName(tableNamePattern, "", 
+				quotedId , conn.isNoBackslashEscapesSet());
+		//There *should* be 2 rows, if any.
+		if (parseList.size() == 2) {
+			tableNamePat = (String) parseList.get(1);
+		} else {
+			tableNamePat = tableNamePattern;
+		}
+		
 		java.sql.PreparedStatement pStmt = null;
 
 		String sql = "SELECT TABLE_SCHEMA AS TABLE_CAT, "
@@ -1485,7 +1494,7 @@ public class DatabaseMetaDataUsingInfoSchema extends DatabaseMetaData {
 				pStmt.setString(1, "%");
 			}
 			
-			pStmt.setString(2, tableNamePattern);
+			pStmt.setString(2, tableNamePat);
 
 			// This overloading of IN (...) allows us to cache this
 			// prepared statement
@@ -1530,6 +1539,10 @@ public class DatabaseMetaDataUsingInfoSchema extends DatabaseMetaData {
 			}
 		}
 	}
+	
+	public boolean gethasParametersView() {
+		return this.hasParametersView;
+	}
 
 
 }
