@@ -1,25 +1,23 @@
 /*
   Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
-  The MySQL Connector/J is licensed under the terms of the GPL,
-  like most MySQL Connectors. There are special exceptions to the
-  terms and conditions of the GPL as it is applied to this software,
-  see the FLOSS License Exception available on mysql.com.
+  The MySQL Connector/J is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
+  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
+  this software, see the FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License as
-  published by the Free Software Foundation; version 2 of the
-  License.
+  This program is free software; you can redistribute it and/or modify it under the terms
+  of the GNU General Public License as published by the Free Software Foundation; version 2
+  of the License.
 
-  This program is distributed in the hope that it will be useful,  
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02110-1301 USA
+  You should have received a copy of the GNU General Public License along with this
+  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
+  Floor, Boston, MA 02110-1301  USA
  
  */
 package com.mysql.jdbc;
@@ -42,7 +40,11 @@ import com.mysql.jdbc.log.Log;
 
 public class LoadBalancedMySQLConnection implements MySQLConnection {
 
-	private LoadBalancingConnectionProxy proxy;
+	protected LoadBalancingConnectionProxy proxy;
+	
+	public LoadBalancingConnectionProxy getProxy(){
+		return this.proxy;
+	}
 
 	protected MySQLConnection getActiveMySQLConnection() {
 		return this.proxy.currentConn;
@@ -2354,9 +2356,6 @@ public class LoadBalancedMySQLConnection implements MySQLConnection {
 		getActiveMySQLConnection().setTransactionIsolation(level);
 	}
 
-	public synchronized void setTypeMap(Map map) throws SQLException {
-		getActiveMySQLConnection().setTypeMap(map);
-	}
 
 	public void shutdownServer() throws SQLException {
 		getActiveMySQLConnection().shutdownServer();
@@ -2481,5 +2480,44 @@ public class LoadBalancedMySQLConnection implements MySQLConnection {
 		
 	}
 	
+	public boolean shouldExecutionTriggerServerSwapAfter(String SQL){
+		return false;
+	}
+
+	public boolean isProxySet() {
+		return this.getActiveMySQLConnection().isProxySet();
+	}
+
+
+	public String getLoadBalanceAutoCommitStatementRegex() {
+		return getActiveMySQLConnection()
+			.getLoadBalanceAutoCommitStatementRegex();
+	}
+
+	public int getLoadBalanceAutoCommitStatementThreshold() {
+		return getActiveMySQLConnection()
+			.getLoadBalanceAutoCommitStatementThreshold();
+	}
+
+	public void setLoadBalanceAutoCommitStatementRegex(
+			String loadBalanceAutoCommitStatementRegex) {
+		getActiveMySQLConnection()
+			.setLoadBalanceAutoCommitStatementRegex(loadBalanceAutoCommitStatementRegex);
+		
+	}
+
+	public void setLoadBalanceAutoCommitStatementThreshold(
+			int loadBalanceAutoCommitStatementThreshold) {
+		getActiveMySQLConnection()
+			.setLoadBalanceAutoCommitStatementThreshold(loadBalanceAutoCommitStatementThreshold);
+		
+	}
+
+	public synchronized void setTypeMap(Map map) throws SQLException {
+		getActiveMySQLConnection().setTypeMap(map);
+	}
+	
+	
+	
 
 }
