@@ -1,29 +1,28 @@
 /*
   Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
 
-  The MySQL Connector/J is licensed under the terms of the GPL,
-  like most MySQL Connectors. There are special exceptions to the
-  terms and conditions of the GPL as it is applied to this software,
-  see the FLOSS License Exception available on mysql.com.
+  The MySQL Connector/J is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
+  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
+  this software, see the FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License as
-  published by the Free Software Foundation; version 2 of the
-  License.
+  This program is free software; you can redistribute it and/or modify it under the terms
+  of the GNU General Public License as published by the Free Software Foundation; version 2
+  of the License.
 
-  This program is distributed in the hope that it will be useful,  
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02110-1301 USA
+  You should have received a copy of the GNU General Public License along with this
+  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
+  Floor, Boston, MA 02110-1301  USA
  
  */
 package com.mysql.jdbc;
 
+import java.lang.reflect.Constructor;
 import java.lang.reflect.InvocationHandler;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
@@ -71,6 +70,7 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 	private long lastUsed = 0;
 	private long transactionCount = 0;
 	private ConnectionGroup connectionGroup = null;
+	private String closedReason = null;
 
 	public static final String BLACKLIST_TIMEOUT_PROPERTY_KEY = "loadBalanceBlacklistTimeout";
 
@@ -148,6 +148,27 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 	private Map<Class, Boolean> jdbcInterfacesForProxyCache = new HashMap<Class, Boolean>();
 	
 	private MySQLConnection thisAsConnection = null;
+
+	private int autoCommitSwapThreshold = 0;
+	
+	private static Constructor JDBC_4_LB_CONNECTION_CTOR;
+	
+	static {
+		if(Util.isJdbc4()){
+			try {
+				JDBC_4_LB_CONNECTION_CTOR =  Class.forName(
+						"com.mysql.jdbc.JDBC4LoadBalancedMySQLConnection").getConstructor(
+						new Class[] { LoadBalancingConnectionProxy.class});
+			} catch (SecurityException e) {
+				throw new RuntimeException(e);
+			} catch (NoSuchMethodException e) {
+				throw new RuntimeException(e);
+			} catch (ClassNotFoundException e) {
+				throw new RuntimeException(e);
+			}
+		}
+	}
+	
 	
 	
 
@@ -259,6 +280,39 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 					strategy, "InvalidLoadBalanceStrategy", null).get(0);
 		}
 
+		String autoCommitSwapThresholdAsString = props.getProperty("loadBalanceAutoCommitStatementThreshold",
+		"0");
+		try {
+			this.autoCommitSwapThreshold  = Integer.parseInt(autoCommitSwapThresholdAsString);
+		} catch (NumberFormatException nfe) {
+			throw SQLError.createSQLException(Messages.getString(
+					"LoadBalancingConnectionProxy.badValueForLoadBalanceAutoCommitStatementThreshold",
+					new Object[] { autoCommitSwapThresholdAsString }),
+					SQLError.SQL_STATE_ILLEGAL_ARGUMENT, null);
+		}
+		
+		String autoCommitSwapRegex = props.getProperty("loadBalanceAutoCommitStatementRegex","");
+		if(!("".equals(autoCommitSwapRegex))){
+			try{
+				"".matches(autoCommitSwapRegex);
+			} catch (Exception e){
+				throw SQLError.createSQLException(Messages.getString(
+						"LoadBalancingConnectionProxy.badValueForLoadBalanceAutoCommitStatementRegex",
+						new Object[] { autoCommitSwapRegex }),
+						SQLError.SQL_STATE_ILLEGAL_ARGUMENT, null);
+			}
+		}
+		
+		if(this.autoCommitSwapThreshold > 0){
+			String statementInterceptors = this.localProps.getProperty("statementInterceptors");
+			if(statementInterceptors == null){
+				this.localProps.setProperty("statementInterceptors", "com.mysql.jdbc.LoadBalancedAutoCommitInterceptor");
+			} else if(statementInterceptors.length() > 0){
+				this.localProps.setProperty("statementInterceptors", statementInterceptors + ",com.mysql.jdbc.LoadBalancedAutoCommitInterceptor");
+			}
+			props.setProperty("statementInterceptors", this.localProps.getProperty("statementInterceptors"));
+			
+		}
 		this.balancer.init(null, props);
 		
 
@@ -266,13 +320,15 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 				lbExceptionChecker, "InvalidLoadBalanceExceptionChecker", null).get(0);
 		this.exceptionChecker.init(null, props);
 
-		thisAsConnection = new LoadBalancedMySQLConnection(this);
-
-		
+		if(Util.isJdbc4()  || JDBC_4_LB_CONNECTION_CTOR != null){
+			thisAsConnection =  (MySQLConnection) Util.handleNewInstance(JDBC_4_LB_CONNECTION_CTOR,
+					new Object[] {this}, null);
+		}else{
+			thisAsConnection = new LoadBalancedMySQLConnection(this);
+		}
 		pickNewConnection();
 		
 		
-		
 	}
 
 	/**
@@ -417,13 +473,18 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		}
 
 	}
+	
+		
+	public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
+		return this.invoke(proxy, method, args, true);
+	}
 
 	/**
 	 * Proxies method invocation on the java.sql.Connection interface, trapping
 	 * "close", "isClosed" and "commit/rollback" (to switch connections for load
 	 * balancing).
 	 */
-	public Object invoke(Object proxy, Method method, Object[] args)
+	public Object invoke(Object proxy, Method method, Object[] args, boolean swapAtTransactionBoundary)
 			throws Throwable {
 
 		String methodName = method.getName();
@@ -446,6 +507,7 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		if ("close".equals(methodName)) {
 			closeAllConnections();
 			this.isClosed = true;
+			this.closedReason = "Connection explicitly closed.";
 			return null;
 		}
 
@@ -454,8 +516,12 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		}
 
 		if (this.isClosed) {
+			String reason = "No operations allowed after connection closed.";
+			if(this.closedReason != null){
+				reason += ("  " + this.closedReason);
+			}
 			throw SQLError.createSQLException(
-					"No operations allowed after connection closed.",
+					reason,
 					SQLError.SQL_STATE_CONNECTION_NOT_OPEN, null /*
 																 * no access to
 																 * a interceptor
@@ -485,7 +551,7 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		} catch (InvocationTargetException e) {
 			dealWithInvocationException(e);
 		} finally {
-			if ("commit".equals(methodName) || "rollback".equals(methodName)) {
+			if (swapAtTransactionBoundary && ("commit".equals(methodName) || "rollback".equals(methodName))) {
 				this.inTransaction = false;
 
 				// Update stats
@@ -530,10 +596,9 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 			invalidateCurrentConnection();
 		}
 
-		boolean currentAutoCommit = this.currentConn.getAutoCommit();
-		int currentTransIsolation = this.currentConn.getTransactionIsolation();
 		int pingTimeout = this.currentConn.getLoadBalancePingTimeout();
 		boolean pingBeforeReturn = this.currentConn.getLoadBalanceValidateConnectionOnSwapServer();
+		
 		for(int hostsTried = 0, hostsToTry = this.hostList.size(); hostsTried <= hostsToTry; hostsTried++){
 			try{
 				ConnectionImpl newConn = this.balancer.pickConnection(
@@ -550,8 +615,8 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 							newConn.pingInternal(true, pingTimeout);
 						}
 					}
-					newConn.setTransactionIsolation(currentTransIsolation);
-					newConn.setAutoCommit(currentAutoCommit);
+					
+					syncSessionState(this.currentConn, newConn);
 				}
 				
 				this.currentConn = newConn;
@@ -568,6 +633,8 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		}
 		// no hosts available to swap connection to, close up.
 		this.isClosed = true;
+		this.closedReason = "Connection closed after inability to pick valid new connection during fail-over.";
+
 	}
 
 	/**
@@ -707,6 +774,7 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 						// pool won't do it
 						closeAllConnections();
 						this.isClosed = true;
+						this.closedReason = "Connection closed because ping of current connection failed.";
 						throw e;
 					}
 
@@ -738,6 +806,7 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 		if (!foundHost) {
 			closeAllConnections();
 			this.isClosed = true;
+			this.closedReason = "Connection closed due to inability to ping any active connections.";
 			// throw the stored Exception, if exists
 			if (se != null) {
 				throw se;
@@ -928,11 +997,23 @@ public class LoadBalancingConnectionProxy implements InvocationHandler,
 	
 	public long getCurrentTransactionDuration(){
 		long st = 0;
+		
 		if(this.inTransaction && (st = this.transactionStartTime) > 0){
 			return getLocalTimeBestResolution() - this.transactionStartTime;
 		}
 		return 0;
 	}
-		
 	
+	protected void syncSessionState(Connection initial, Connection target)
+			throws SQLException {
+		if (initial == null || target == null) {
+			return;
+		}
+		target.setAutoCommit(initial.getAutoCommit());
+		target.setCatalog(initial.getCatalog());
+		target.setTransactionIsolation(initial.getTransactionIsolation());
+		target.setReadOnly(initial.isReadOnly());
+
+	}
+
 }
\ No newline at end of file
