@@ -1,26 +1,24 @@
 /*
- Copyright  2002-2004 MySQL AB, 2008-2010 Sun Microsystems
- All rights reserved. Use is subject to license terms.
+ Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
+ 
 
-  The MySQL Connector/J is licensed under the terms of the GPL,
-  like most MySQL Connectors. There are special exceptions to the
-  terms and conditions of the GPL as it is applied to this software,
-  see the FLOSS License Exception available on mysql.com.
+  The MySQL Connector/J is licensed under the terms of the GPLv2
+  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
+  There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
+  this software, see the FLOSS License Exception
+  <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License as
-  published by the Free Software Foundation; version 2 of the
-  License.
+  This program is free software; you can redistribute it and/or modify it under the terms
+  of the GNU General Public License as published by the Free Software Foundation; version 2
+  of the License.
 
-  This program is distributed in the hope that it will be useful,  
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the GNU General Public License for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-  02110-1301 USA
+  You should have received a copy of the GNU General Public License along with this
+  program; if not, write to the Free Software Foundation, Inc., 51 Franklin St, Fifth
+  Floor, Boston, MA 02110-1301  USA
 
 
 
@@ -1002,7 +1000,7 @@ public class StringUtils {
 
 			if (allowBackslashEscapes && c == '\\') {
 				escaped = !escaped;
-			} else if (c == markerCloses.charAt(markerTypeFound) && !escaped) {
+			} else if (contextMarker != Character.MIN_VALUE && c == markerCloses.charAt(markerTypeFound) && !escaped) {
 				contextMarker = Character.MIN_VALUE;
 			} else if ((ind = marker.indexOf(c)) != -1 && !escaped
 					&& contextMarker == Character.MIN_VALUE) {
@@ -1109,10 +1107,10 @@ public class StringUtils {
 	 * @throws IllegalArgumentException
 	 *             DOCUMENT ME!
 	 */
-	public static final List split(String stringToSplit, String delimiter,
+	public static final List<String> split(String stringToSplit, String delimiter,
 			String markers, String markerCloses, boolean trim) {
 		if (stringToSplit == null) {
-			return new ArrayList();
+			return new ArrayList<String>();
 		}
 
 		if (delimiter == null) {
@@ -1122,7 +1120,7 @@ public class StringUtils {
 		int delimPos = 0;
 		int currentPos = 0;
 
-		List splitTokens = new ArrayList();
+		List<String> splitTokens = new ArrayList<String>();
 
 		while ((delimPos = indexOfIgnoreCaseRespectMarker(currentPos,
 				stringToSplit, delimiter, markers, markerCloses, false)) != -1) {
@@ -1681,6 +1679,89 @@ public class StringUtils {
 		return buf.toString();
 	}
 	
+	/**
+	 * Next two functions are to help DBMD check if 
+	 * the given string is in form of database.name and return it 
+	 * as "database";"name" with comments removed.
+	 * If string is NULL or wildcard (%), returns null and exits.
+	 * 
+	 * First, we sanitize...
+	 * 
+	 * @param src
+	 *            the source string
+	 * @return the input string with all comment-delimited data removed
+	 */
+	public static String sanitizeProcOrFuncName(String src) {
+		if ((src == null) || (src == "%")) {
+			return null;
+		} else {
+			return src;
+		}
+	}
+
+	/**
+	 * Next we check if there is anything to split. If so 
+	 * we return result in form of "database";"name"
+	 * If string is NULL or wildcard (%), returns null and exits.
+	 * 
+	 * @param src
+	 *            the source string
+	 * @param cat
+	 *            Catalog, if available
+	 * @param quotId
+	 *            quoteId as defined on server
+	 * @param isNoBslashEscSet
+	 *            Is our connection in BackSlashEscape mode
+	 * @return the input string with all comment-delimited data removed
+	 */
+	public static List splitDBdotName(String src, String cat, String quotId,
+			boolean isNoBslashEscSet) {
+		if ((src == null) || (src == "%")) {
+			return new ArrayList();
+		}
+		String retval = src;
+		//So we have a name to check, meaning more actual processing
+		String tmpCat = cat;
+		//This is NOT enough. I.e., what if database is named `MyDatabase 1.0.0`... thus trueDotIndex
+		int trueDotIndex = -1;
+		if (!" ".equals(quotId)) {
+			//Presumably, if there is a database name attached and it contains dots, then it should
+			//be quoted so we first check for that
+			trueDotIndex = StringUtils.indexOfIgnoreCaseRespectQuotes(0,
+					retval, quotId + ".", quotId.charAt(0), !isNoBslashEscSet);
+			if (trueDotIndex == -1){
+				trueDotIndex = StringUtils.indexOfIgnoreCaseRespectQuotes(0,
+						retval, ".", quotId.charAt(0), !isNoBslashEscSet);
+			}
+		} else {
+			trueDotIndex = retval.indexOf(".");
+		}
+
+		List retTokens = new ArrayList(2);
+
+		//trueDotIndex being still -1 means there's no
+		//catalog attached!
+		if (trueDotIndex != -1) {
+			//There is a catalog attached
+			tmpCat = retval.substring(0, trueDotIndex);
+			if (StringUtils.startsWithIgnoreCaseAndWs(tmpCat, quotId) && tmpCat.trim().endsWith(quotId)) {
+				tmpCat = tmpCat.substring(1, tmpCat.length() - 1);
+			}
+
+			retval = retval.substring(trueDotIndex + 1);
+			retval = new String(StringUtils.stripEnclosure(retval
+				.getBytes(), quotId, quotId));
+		} else {
+			//No catalog attached, strip retval and return
+			retval = new String(StringUtils.stripEnclosure(retval
+				.getBytes(), quotId, quotId));
+		}
+		
+		retTokens.add(tmpCat);
+		retTokens.add(retval);
+		return retTokens;
+	}
+	
 	public static final boolean isEmptyOrWhitespaceOnly(String str) {
 		if (str == null || str.length() == 0) {
 			return true;
@@ -1696,4 +1777,36 @@ public class StringUtils {
 		
 		return true;
 	}
+	
+	public static String escapeQuote(String src, String quotChar) {
+		if (src == null) {
+			return null;
+		}
+
+		src = new String(stripEnclosure(src.getBytes(), quotChar, quotChar));
+
+		int lastNdx = src.indexOf(quotChar);
+		String tmpSrc;
+		String tmpRest;
+
+		tmpSrc = src.substring(0, lastNdx);
+		tmpSrc = tmpSrc + quotChar + quotChar;
+		
+		tmpRest = src.substring(lastNdx+1, src.length());
+		
+		lastNdx = tmpRest.indexOf(quotChar);
+		while (lastNdx > -1) {
+			
+			tmpSrc = tmpSrc + tmpRest.substring(0, lastNdx);
+			tmpSrc = tmpSrc + quotChar + quotChar;
+			tmpRest = tmpRest.substring(lastNdx+1, tmpRest.length());
+			
+			lastNdx = tmpRest.indexOf(quotChar);
+		}
+		tmpSrc = tmpSrc + tmpRest;
+		src = tmpSrc;
+
+		return src;
+	}
+	
 }
